@@ -6,7 +6,7 @@ from behave import step, then
 import patroni.psycopg as pg
 
 
-@step('I create a logical replication slot {slot_name} on {pg_name:w} with the {plugin:w} plugin')
+@step('I create a logical replication slot {slot_name} on {pg_name:name} with the {plugin:w} plugin')
 def create_logical_replication_slot(context, slot_name, pg_name, plugin):
     try:
         output = context.pctl.query(pg_name, ("SELECT pg_create_logical_replication_slot('{0}', '{1}'),"
@@ -17,9 +17,9 @@ def create_logical_replication_slot(context, slot_name, pg_name, plugin):
         assert False, "Error creating slot {0} on {1} with plugin {2}".format(slot_name, pg_name, plugin)
 
 
-@step('{pg_name:w} has a logical replication slot named {slot_name}'
+@step('{pg_name:name} has a logical replication slot named {slot_name}'
       ' with the {plugin:w} plugin after {time_limit:d} seconds')
-@then('{pg_name:w} has a logical replication slot named {slot_name}'
+@then('{pg_name:name} has a logical replication slot named {slot_name}'
       ' with the {plugin:w} plugin after {time_limit:d} seconds')
 def has_logical_replication_slot(context, pg_name, slot_name, plugin, time_limit):
     time_limit *= context.timeout_multiplier
@@ -38,8 +38,8 @@ def has_logical_replication_slot(context, pg_name, slot_name, plugin, time_limit
     assert False, f"Error looking for slot {slot_name} on {pg_name} with plugin {plugin}"
 
 
-@step('{pg_name:w} does not have a replication slot named {slot_name:w}')
-@then('{pg_name:w} does not have a replication slot named {slot_name:w}')
+@step('{pg_name:name} does not have a replication slot named {slot_name:w}')
+@then('{pg_name:name} does not have a replication slot named {slot_name:w}')
 def does_not_have_replication_slot(context, pg_name, slot_name):
     try:
         row = context.pctl.query(pg_name, ("SELECT 1 FROM pg_replication_slots"
@@ -49,7 +49,8 @@ def does_not_have_replication_slot(context, pg_name, slot_name):
         assert False, "Error looking for slot {0} on {1}".format(slot_name, pg_name)
 
 
-@step('{slot_type:w} slot {slot_name:w} is in sync between {pg_name1:w} and {pg_name2:w} after {time_limit:d} seconds')
+@step('{slot_type:w} slot {slot_name:w} is in sync between '
+      '{pg_name1:name} and {pg_name2:name} after {time_limit:d} seconds')
 def slots_in_sync(context, slot_type, slot_name, pg_name1, pg_name2, time_limit):
     time_limit *= context.timeout_multiplier
     max_time = time.time() + int(time_limit)
@@ -68,17 +69,17 @@ def slots_in_sync(context, slot_type, slot_name, pg_name1, pg_name2, time_limit)
         f"{slot_type} slot {slot_name} is not in sync between {pg_name1} and {pg_name2} after {time_limit} seconds"
 
 
-@step('I get all changes from logical slot {slot_name:w} on {pg_name:w}')
+@step('I get all changes from logical slot {slot_name:w} on {pg_name:name}')
 def logical_slot_get_changes(context, slot_name, pg_name):
     context.pctl.query(pg_name, "SELECT * FROM pg_logical_slot_get_changes('{0}', NULL, NULL)".format(slot_name))
 
 
-@step('I get all changes from physical slot {slot_name:w} on {pg_name:w}')
+@step('I get all changes from physical slot {slot_name:w} on {pg_name:name}')
 def physical_slot_get_changes(context, slot_name, pg_name):
     context.pctl.query(pg_name, f"SELECT * FROM pg_replication_slot_advance('{slot_name}', pg_current_wal_lsn())")
 
 
-@step('{pg_name:w} has a physical replication slot named {slot_name} after {time_limit:d} seconds')
+@step('{pg_name:name} has a physical replication slot named {slot_name} after {time_limit:d} seconds')
 def has_physical_replication_slot(context, pg_name, slot_name, time_limit):
     time_limit *= context.timeout_multiplier
     max_time = time.time() + int(time_limit)
@@ -94,13 +95,33 @@ def has_physical_replication_slot(context, pg_name, slot_name, time_limit):
     assert False, f"Physical slot {slot_name} doesn't exist after {time_limit} seconds"
 
 
-@step('"{name}" key in DCS has {subkey:w} in {key:w}')
+@step('physical replication slot named {slot_name} on {pg_name:name} has no xmin value after {time_limit:d} seconds')
+def physical_slot_no_xmin(context, pg_name, slot_name, time_limit):
+    time_limit *= context.timeout_multiplier
+    max_time = time.time() + int(time_limit)
+    query = "SELECT xmin FROM pg_catalog.pg_replication_slots WHERE slot_type = 'physical'"
+    f" AND slot_name = '{slot_name}'"
+    exists = False
+    while time.time() < max_time:
+        try:
+            row = context.pctl.query(pg_name, query).fetchone()
+            exists = bool(row)
+            if exists and row[0] is None:
+                return
+        except Exception:
+            pass
+        time.sleep(1)
+    assert False, f"Physical slot {slot_name} doesn't exist after {time_limit} seconds" if not exists \
+        else f"Physical slot {slot_name} has xmin value after {time_limit} seconds"
+
+
+@step('"{name}" key in DCS has {subkey} in {key:w}')
 def dcs_key_contains(context, name, subkey, key):
     response = json.loads(context.dcs_ctl.query(name))
     assert key in response and subkey in response[key], f"{name} key in DCS doesn't have {subkey} in {key}"
 
 
-@step('"{name}" key in DCS does not have {subkey:w} in {key:w}')
+@step('"{name}" key in DCS does not have {subkey} in {key:w}')
 def dcs_key_does_not_contain(context, name, subkey, key):
     response = json.loads(context.dcs_ctl.query(name))
     assert key not in response or subkey not in response[key], f"{name} key in DCS has {subkey} in {key}"
