@@ -3,7 +3,7 @@ import re
 import time
 
 from copy import deepcopy
-from typing import Collection, List, NamedTuple, Tuple, TYPE_CHECKING
+from typing import Collection, List, NamedTuple, Optional, TYPE_CHECKING
 
 from .. import global_config
 from ..collections import CaseInsensitiveDict, CaseInsensitiveSet
@@ -144,7 +144,7 @@ def parse_sync_standby_names(value: str) -> _SSN:
             if len(synclist) == i + 1:  # except the last token
                 raise ValueError("Unparsable synchronous_standby_names value %r: Unexpected token %s %r at %d" %
                                  (value, a_type, a_value, a_pos))
-            elif a_type != 'comma':
+            if a_type != 'comma':
                 raise ValueError("Unparsable synchronous_standby_names value %r: ""Got token %s %r while"
                                  " expecting comma at %d" % (value, a_type, a_value, a_pos))
         elif a_type in {'ident', 'first', 'any'}:
@@ -160,6 +160,26 @@ def parse_sync_standby_names(value: str) -> _SSN:
     return _SSN(sync_type, has_star, num, members)
 
 
+class _SyncState(NamedTuple):
+    """Class representing the current synchronous state.
+
+    :ivar sync_type: possible values: ``off``, ``priority``, ``quorum``
+    :ivar numsync: how many nodes are required to be synchronous (according to ``synchronous_standby_names``).
+                   Is ``0`` if ``synchronous_standby_names`` value is invalid or contains ``*``.
+    :ivar numsync_confirmed: how many nodes are known to be synchronous according to the ``pg_stat_replication`` view.
+                             Only nodes that caught up with the :attr:`SyncHandler._primary_flush_lsn` are counted.
+    :ivar sync: collection of synchronous node names. In case of quorum commit all nodes listed
+                in ``synchronous_standby_names``, otherwise nodes that are confirmed to be synchronous according
+                to the ``pg_stat_replication`` view.
+    :ivar active: collection of node names that are streaming and have no restrictions to become synchronous.
+    """
+    sync_type: str
+    numsync: int
+    numsync_confirmed: int
+    sync: CaseInsensitiveSet
+    active: CaseInsensitiveSet
+
+
 class _Replica(NamedTuple):
     """Class representing a single replica that is eligible to be synchronous.
 
@@ -223,6 +243,8 @@ class _ReplicaList(List[_Replica]):
         # Prefer replicas that are in state ``sync`` and with higher values of ``write``/``flush``/``replay`` LSN.
         self.sort(key=lambda r: (r.sync_state, r.lsn), reverse=True)
 
+        # When checking ``maximum_lag_on_syncnode`` we want to compare with the most
+        # up-to-date replica otherwise with cluster LSN if there is only one replica.
         self.max_lsn = max(self, key=lambda x: x.lsn).lsn if len(self) > 1 else postgresql.last_operation()
 
 
@@ -284,12 +306,22 @@ END;$$""")
             # if standby name is listed in the /sync key we can count it as synchronous, otherwise
             # it becomes really synchronous when sync_state = 'sync' and it is known that it managed to catch up
             if replica.application_name not in self._ready_replicas\
-                    and replica.application_name in self._ssn_data.members\
-                    and (cluster.sync.matches(replica.application_name)
-                         or replica.sync_state == 'sync' and replica.lsn >= self._primary_flush_lsn):
-                self._ready_replicas[replica.application_name] = replica.pid
+                    and replica.application_name in self._ssn_data.members:
+                if global_config.is_quorum_commit_mode:
+                    # When quorum commit is enabled we can't check against cluster.sync because nodes
+                    # are written there when at least one of them caught up with _primary_flush_lsn.
+                    if replica.lsn >= self._primary_flush_lsn\
+                            and (replica.sync_state == 'quorum'
+                                 or (not self._postgresql.supports_quorum_commit
+                                     and replica.sync_state in ('sync', 'potential'))):
+                        self._ready_replicas[replica.application_name] = replica.pid
+                elif cluster.sync.matches(replica.application_name)\
+                        or replica.sync_state == 'sync' and replica.lsn >= self._primary_flush_lsn:
+                    # if standby name is listed in the /sync key we can count it as synchronous, otherwise it becomes
+                    # "really" synchronous when sync_state = 'sync' and we known that it managed to catch up
+                    self._ready_replicas[replica.application_name] = replica.pid
 
-    def current_state(self, cluster: Cluster) -> Tuple[CaseInsensitiveSet, CaseInsensitiveSet]:
+    def current_state(self, cluster: Cluster) -> _SyncState:
         """Find the best candidates to be the synchronous standbys.
 
         Current synchronous standby is always preferred, unless it has disconnected or does not want to be a
@@ -297,51 +329,86 @@ END;$$""")
 
         Standbys are selected based on values from the global configuration:
 
-        - `maximum_lag_on_syncnode`: would help swapping unhealthy sync replica in case if it stops
-          responding (or hung). Please set the value high enough so it won't unncessarily swap sync
-          standbys during high loads. Any value less or equal of 0 keeps the behavior backward compatible.
-          Please note that it will not also swap sync standbys in case where all replicas are hung.
-        - `synchronous_node_count`: controlls how many nodes should be set as synchronous.
+            - ``maximum_lag_on_syncnode``: would help swapping unhealthy sync replica in case it stops
+              responding (or hung). Please set the value high enough, so it won't unnecessarily swap sync
+              standbys during high loads. Any value less or equal to ``0`` keeps the behavior backwards compatible.
+              Please note that it will also not swap sync standbys when all replicas are hung.
 
-        :returns: tuple of candidates :class:`CaseInsensitiveSet` and synchronous standbys :class:`CaseInsensitiveSet`.
+            - ``synchronous_node_count``: controls how many nodes should be set as synchronous.
+
+        :param cluster: current cluster topology from DCS
+
+        :returns: current synchronous replication state as a :class:`_SyncState` object
         """
         self._handle_synchronous_standby_names_change()
 
         replica_list = _ReplicaList(self._postgresql, cluster)
         self._process_replica_readiness(cluster, replica_list)
 
+        active = CaseInsensitiveSet()
+        sync_nodes = CaseInsensitiveSet()
+        numsync_confirmed = 0
+
         sync_node_count = global_config.synchronous_node_count if self._postgresql.supports_multiple_sync else 1
         sync_node_maxlag = global_config.maximum_lag_on_syncnode
 
-        candidates = CaseInsensitiveSet()
-        sync_nodes = CaseInsensitiveSet()
         # Prefer members without nofailover tag. We are relying on the fact that sorts are guaranteed to be stable.
         for replica in sorted(replica_list, key=lambda x: x.nofailover):
             if sync_node_maxlag <= 0 or replica_list.max_lsn - replica.lsn <= sync_node_maxlag:
-                candidates.add(replica.application_name)
-                if replica.sync_state == 'sync' and replica.application_name in self._ready_replicas:
-                    sync_nodes.add(replica.application_name)
-            if len(candidates) >= sync_node_count:
-                break
+                if global_config.is_quorum_commit_mode:
+                    # We do not add nodes with `nofailover` enabled because that reduces availability.
+                    # We need to check LSN quorum only among nodes that are promotable because
+                    # there is a chance that a non-promotable node is ahead of a promotable one.
+                    if not replica.nofailover or len(active) < sync_node_count:
+                        if replica.application_name in self._ready_replicas:
+                            numsync_confirmed += 1
+                        active.add(replica.application_name)
+                else:
+                    active.add(replica.application_name)
+                    if replica.sync_state == 'sync' and replica.application_name in self._ready_replicas:
+                        sync_nodes.add(replica.application_name)
+                        numsync_confirmed += 1
+                    if len(active) >= sync_node_count:
+                        break
 
-        return candidates, sync_nodes
+        if global_config.is_quorum_commit_mode:
+            sync_nodes = CaseInsensitiveSet() if self._ssn_data.has_star else self._ssn_data.members
 
-    def set_synchronous_standby_names(self, sync: Collection[str]) -> None:
-        """Constructs and sets "synchronous_standby_names" GUC value.
+        return _SyncState(
+            self._ssn_data.sync_type,
+            0 if self._ssn_data.has_star else self._ssn_data.num,
+            numsync_confirmed,
+            sync_nodes,
+            active)
+
+    def set_synchronous_standby_names(self, sync: Collection[str], num: Optional[int] = None) -> None:
+        """Constructs and sets ``synchronous_standby_names`` GUC value.
+
+        .. note::
+            standbys in ``synchronous_standby_names`` will be sorted by name.
 
         :param sync: set of nodes to sync to
+        :param num: specifies number of nodes to sync to. The *num* is set only in case if quorum commit is enabled
         """
-        has_asterisk = '*' in sync
+        # Special case. If sync nodes set is empty but requested num of sync nodes >= 1
+        # we want to set synchronous_standby_names to '*'
+        has_asterisk = '*' in sync or num and num >= 1 and not sync
         if has_asterisk:
             sync = ['*']
         else:
-            sync = [quote_standby_name(x) for x in sync]
+            sync = [quote_standby_name(x) for x in sorted(sync)]
 
         if self._postgresql.supports_multiple_sync and len(sync) > 1:
-            sync_param = '{0} ({1})'.format(len(sync), ','.join(sync))
+            if num is None:
+                num = len(sync)
+            sync_param = ','.join(sync)
         else:
             sync_param = next(iter(sync), None)
 
+        if global_config.is_quorum_commit_mode and sync or self._postgresql.supports_multiple_sync and len(sync) > 1:
+            prefix = 'ANY ' if global_config.is_quorum_commit_mode and self._postgresql.supports_quorum_commit else ''
+            sync_param = f'{prefix}{num} ({sync_param})'
+
         if not (self._postgresql.config.set_synchronous_standby_names(sync_param)
                 and self._postgresql.state == 'running' and self._postgresql.is_primary()) or has_asterisk:
             return
