@@ -9,13 +9,24 @@ import os
 import shutil
 import socket
 
-from typing import Any, cast, Dict, Union, Iterator, List, Optional as OptionalType, Tuple, TYPE_CHECKING
+from typing import Any, cast, Dict, Iterator, List, Optional as OptionalType, Tuple, TYPE_CHECKING, Union
 
 from .collections import CaseInsensitiveSet, EMPTY_DICT
 from .dcs import dcs_modules
 from .exceptions import ConfigParseError
-from .utils import parse_int, split_host_port, data_directory_is_empty, get_major_version
 from .log import type_logformat
+from .utils import data_directory_is_empty, get_major_version, parse_int, split_host_port
+
+# Additional parameters to fine-tune validation process
+_validation_params: Dict[str, Any] = {}
+
+
+def populate_validate_params(ignore_listen_port: bool = False) -> None:
+    """Populate parameters used to fine-tune the validation of the Patroni config.
+
+    :param ignore_listen_port: ignore the bind failures for the ports marked as `listen`.
+    """
+    _validation_params['ignore_listen_port'] = ignore_listen_port
 
 
 def validate_log_field(field: Any) -> bool:
@@ -139,7 +150,8 @@ def validate_host_port(host_port: str, listen: bool = False, multiple_hosts: boo
             s = socket.socket(proto[0][0], socket.SOCK_STREAM)
             try:
                 if s.connect_ex((host, port)) == 0:
-                    if listen:
+                    # Do not raise an exception if ignore_listen_port is set to True.
+                    if listen and not _validation_params.get('ignore_listen_port', False):
                         raise ConfigParseError("Port {} is already in use.".format(port))
                 elif not listen:
                     raise ConfigParseError("{} is not reachable".format(host_port))
@@ -988,6 +1000,7 @@ schema = Schema({
         Optional("dir"): str,
         Optional("file_num"): int,
         Optional("file_size"): int,
+        Optional("mode"): IntValidator(min=0, max=511, expected_type=int, raise_assert=True),
         Optional("loggers"): dict
     },
     Optional("ctl"): {
@@ -1024,6 +1037,7 @@ schema = Schema({
             Optional("retry_timeout"): IntValidator(min=3, raise_assert=True),
             Optional("maximum_lag_on_failover"): IntValidator(min=0, raise_assert=True),
             Optional("maximum_lag_on_syncnode"): IntValidator(min=-1, raise_assert=True),
+            Optional('member_slots_ttl'): IntValidator(min=0, base_unit='s', raise_assert=True),
             Optional("postgresql"): {
                 Optional("parameters"): {
                     Optional("max_connections"): IntValidator(1, 262143, raise_assert=True),
