@@ -1,12 +1,16 @@
-import consul
 import unittest
-from unittest.mock import Mock, PropertyMock, patch
+
+from unittest.mock import Mock, patch, PropertyMock
+
+import consul
 
 from consul import ConsulException, NotFound
+
 from patroni.dcs import get_dcs
-from patroni.dcs.consul import AbstractDCS, Cluster, Consul, ConsulInternalError, \
-    ConsulError, ConsulClient, HTTPClient, InvalidSessionTTL, InvalidSession, RetryFailedError
+from patroni.dcs.consul import AbstractDCS, Cluster, Consul, ConsulClient, ConsulError, \
+    ConsulInternalError, HTTPClient, InvalidSession, InvalidSessionTTL, RetryFailedError
 from patroni.postgresql.mpp import get_mpp
+
 from . import SleepException
 
 
@@ -42,8 +46,8 @@ def kv_get(self, key, **kwargs):
                   'ModifyIndex': 6429, 'Value': b'{"leader": "leader", "sync_standby": null}'},
                  {'CreateIndex': 1085, 'Flags': 0, 'Key': key + 'failsafe', 'LockIndex': 0,
                   'ModifyIndex': 6429, 'Value': b'{'},
-                 {'CreateIndex': 1085, 'Flags': 0, 'Key': key + 'status', 'LockIndex': 0,
-                  'ModifyIndex': 6429, 'Value': b'{"optime":4496294792, "slots":{"ls":12345}}'}])
+                 {'CreateIndex': 1085, 'Flags': 0, 'Key': key + 'status', 'LockIndex': 0, 'ModifyIndex': 6429,
+                  'Value': b'{"optime":4496294792,"slots":{"ls":12345},"retain_slots":["postgresql0","postgresql1"]}'}])
     if key == 'service/good/':
         return good_cls
     if key == 'service/broken/':
@@ -263,9 +267,8 @@ class TestConsul(unittest.TestCase):
         d['state'] = 'running'
         d['role'] = 'bla'
         self.assertIsNone(self.c.update_service({}, d))
-        for role in ('master', 'primary'):
-            d['role'] = role
-            self.assertTrue(self.c.update_service({}, d))
+        d['role'] = 'primary'
+        self.assertTrue(self.c.update_service({}, d))
 
     @patch.object(KV, 'put', Mock(side_effect=ConsulException))
     def test_reload_config(self):
